package com.cultural.heritage.controller.good;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonDelBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.appointment.AppointmentDateAddRequest;
import com.cultural.heritage.model.dto.good.GoodAddRequest;
import com.cultural.heritage.model.dto.good.GoodQueryRequest;
import com.cultural.heritage.model.dto.good.GoodUpdateRequest;
import com.cultural.heritage.model.dto.good.service.ServiceGoodAddRequest;
import com.cultural.heritage.model.entity.AppointmentDate;
import com.cultural.heritage.model.entity.AppointmentNumber;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.service.good.AppointmentDateService;
import com.cultural.heritage.service.good.AppointmentNumberService;
import com.cultural.heritage.service.good.GoodService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;


/**
 * 商品接口
 */
@RestController
@RequestMapping("/goods")
@Slf4j
@Tag(name = "商品管理模块")
public class GoodController {


    @Resource
    private GoodService goodService;


    @Resource
    private AppointmentNumberService appointmentNumberService;


    @Resource
    private AppointmentDateService appointmentDateService;


    /**
     * 添加商品
     * @param goodAddRequest  商品添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加商品", description = "参数：商品添加请求体，权限：管理员(admin, boss)，方法名：addGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addGood(@RequestBody GoodAddRequest goodAddRequest) {
        if (goodAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Good good = new Good();
        BeanUtils.copyProperties(goodAddRequest, good);
        boolean save = goodService.save(good);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "商品插入成功");
    }




    /**
     * 添加服务类商品
     * @param serviceGoodAddRequest  服务类商品添加请求体
     * @return 是否添加成功
     */
    @PostMapping("/add/service")
    @Operation(summary = "Web管理员添加服务类商品", description = "参数：服务类商品添加请求体，权限：管理员(admin, boss)，方法名：addServiceGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addServiceGood(@RequestBody ServiceGoodAddRequest serviceGoodAddRequest) {
        if (serviceGoodAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 向商品表插入服务类商品的基本信息
        Good good = new Good();
        BeanUtils.copyProperties(serviceGoodAddRequest, good);
        boolean save = goodService.save(good);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);

        // 向预约人数范围表插入服务类商品的预约人数信息
        Long id = good.getId();
        AppointmentNumber appointmentNumber = new AppointmentNumber();
        BeanUtils.copyProperties(serviceGoodAddRequest, appointmentNumber);
        appointmentNumber.setGoodId(id);
        boolean result = appointmentNumberService.save(appointmentNumber);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);

        // 向预约日期表中批量插入服务类商品的预约信息
        List<AppointmentDateAddRequest> appointmentDateAddRequestList = serviceGoodAddRequest.getAppointmentDateAddRequestList();
        List<AppointmentDate> appointmentDates = appointmentDateAddRequestList.stream().map(appointmentDateAddRequest -> {
            AppointmentDate appointmentDate = new AppointmentDate();
            BeanUtils.copyProperties(appointmentDateAddRequest, appointmentDate);
            appointmentDate.setGoodId(id);
            return appointmentDate;
        }).toList();
        boolean isSuccess = appointmentDateService.saveBatch(appointmentDates);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR);

        return ResultUtils.success(true);
    }



    /**
     * 删除商品
     * @param deleteRequest   商品删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除商品", description = "参数：商品删除请求体，权限：管理员(admin, boss)，方法名：deleteGood")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteGood(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = deleteRequest.getId();
        boolean result = goodService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }



    /**
     * 删除服务类商品
     * @param deleteRequest     服务类商品删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete/service")
    @Operation(summary = "Web端管理员删除服务类商品", description = "参数: 商品删除请求体，权限：管理员(admin, boss)，方法名：deleteServiceGood")
    public BaseResponse<Boolean> deleteServiceGood(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 删除商品表中的服务类商品
        Long id = deleteRequest.getId();
        boolean result = goodService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);

        // 删除预约人数范围表中与该商品关联的记录
        QueryWrapper<AppointmentNumber> numberQueryWrapper = new QueryWrapper<>();
        numberQueryWrapper.eq("goodId", id);
        boolean remove = appointmentNumberService.remove(numberQueryWrapper);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR);

        // 删除预约日期表中与该商品关联的记录
        QueryWrapper<AppointmentDate> dateQueryWrapper = new QueryWrapper<>();
        dateQueryWrapper.eq("goodId", id);
        boolean isSuccess = appointmentDateService.remove(dateQueryWrapper);
        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR);

        return ResultUtils.success(true);
    }


    /**
     * 更新商品
     * @param goodUpdateRequest  商品更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新商品", description = "参数：商品更新请求体，权限：管理员(admin, boss)，方法名：updateGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateGoods(@RequestBody GoodUpdateRequest goodUpdateRequest) {
        if (goodUpdateRequest == null || goodUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Good good = new Good();
        BeanUtils.copyProperties(goodUpdateRequest, good);
        boolean result = goodService.updateById(good);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }




    /**
     * 分页获取商品列表
     * @param goodQueryRequest  查询商品请求体
     * @return  商品信息列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询商品", description = "参数：商品查询请求体，排序字段 sortField: price inventory id，权限：管理员(admin, boss)，方法名：listGoodByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<Good>> listGoodByPage(@RequestBody GoodQueryRequest goodQueryRequest) {
        long current = goodQueryRequest.getCurrent();
        long pageSize = goodQueryRequest.getPageSize();
        QueryWrapper<Good> goodQueryWrapper = goodService.getGoodQueryWrapper(goodQueryRequest);
        Page<Good> page = goodService.page(new Page<>(current, pageSize), goodQueryWrapper);
        return ResultUtils.success(page);
    }




    /**
     * 批量删除商品
     * @param commonDelBatchRequest 批量删除请求体
     * @return 是否批量删除成功
     */
    @PostMapping("/delBatch")
    @Operation(summary = "Web端管理员批量删除商品", description = "参数：商品批量删除请求体，权限：管理员(admin, boss)，方法名：delBatchGoods")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBatchGoods(@RequestBody CommonDelBatchRequest commonDelBatchRequest) {
        List<Integer> idList = commonDelBatchRequest.getIdList();
        boolean result = goodService.removeBatchByIds(idList);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }





    /**
     * 根据id查询商品
     * @param getByIdRequest  根据id查询请求体
     * @return  商品信息
     */
    @PostMapping("/getById")
    @Operation(summary = "根据id查询商品", description = "参数：商品根据id查询请求体，权限：管理员(admin, boss), 方法名:getGoodById")
    public BaseResponse<Good> getGoodById(@RequestBody CommonRequest getByIdRequest) {
        if (getByIdRequest == null || getByIdRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = getByIdRequest.getId();
        Good good = goodService.getById(id);
        ThrowUtils.throwIf(good == null, ErrorCode.NOT_FOUND_ERROR);
        return ResultUtils.success(good);
    }



}
