package com.cultural.heritage.controller.order;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.order.OrderAddRequest;
import com.cultural.heritage.model.dto.order.OrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderUpdateRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.entity.Order;
import com.cultural.heritage.model.entity.OrderItems;
import com.cultural.heritage.model.vo.OrderVO;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("/order")
@Slf4j
@Tag(name = "订单管理模块")
@Transactional(rollbackFor = Exception.class)
public class OrderController {

    @Resource
    private OrderService orderService;



    @Resource
    private OrderItemService orderItemService;


    /**
     * 用户创建订单
     * @param orderMainInfoAddRequest 订单创建请求体
     * @return  是否创建成功
     */
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "小程序端创建用户订单", description = "参数：订单创建请求体，排序字段 sortField: totalAmount，权限：所有人，方法名：addOrder")
    public BaseResponse<Long> addOrder(@RequestBody OrderMainInfoAddRequest orderMainInfoAddRequest) {
        if (orderMainInfoAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 封装订单详细信息请求体
        OrderAddRequest orderAddRequest = orderService.initOrderMainInfo(orderMainInfoAddRequest);
        if (orderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 创建一个总订单
        Order order = new Order();
        BeanUtils.copyProperties(orderAddRequest, order);
        boolean result = orderService.save(order);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        // 创建订单明细
        Long id = order.getId();
        List<OrderItemAddRequest> orderItemList = orderAddRequest.getOrderItemList();
        List<OrderItems> newOrderItemsList = orderItemList.stream().map(orderItem -> {
            OrderItems orderItems = new OrderItems();
            BeanUtils.copyProperties(orderItem, orderItems);
            orderItems.setOrderId(id);
            return orderItems;
        }).toList();
        boolean save = orderItemService.saveBatch(newOrderItemsList);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(id);
    }



    /**
     * 分页查询所有订单(包括订单明细)
     * @param orderQueryRequest  订单查询请求体
     * @return 订单列表信息
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询订单", description = "参数：订单查询请求体，权限：管理员(admin, boss)，方法名：listOrder")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<OrderVO>> listOrder(@RequestBody OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = orderQueryRequest.getCurrent();
        long pageSize = orderQueryRequest.getPageSize();
        Page<Order> page = orderService.page(new Page<>(current, pageSize),
                orderService.getQueryWrapper(orderQueryRequest));
        List<Order> orders = page.getRecords();
        List<OrderVO> orderVOS = orders.stream().map(order -> {
            OrderVO orderVO = new OrderVO();
            BeanUtils.copyProperties(order, orderVO);
            return orderVO;
        }).toList();
        orderVOS.forEach(orderVO -> {
            Long id = orderVO.getId();
            QueryWrapper<OrderItems> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("orderId", id);
            List<OrderItems> list = orderItemService.list(queryWrapper);
            orderVO.setOrderItemList(list);
        });
        Page<OrderVO> orderVOPage = new Page<>();
        orderVOPage.setRecords(orderVOS);
        orderVOPage.setTotal(page.getTotal());
        orderVOPage.setPages(page.getPages());
        orderVOPage.setCurrent(page.getCurrent());
        orderVOPage.setSize(page.getSize());
        return ResultUtils.success(orderVOPage);
    }



    /**
     * 小程序端用户查询订单
     * @param commonRequest  用户id请求体
     * @return  用户订单列表
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询订单", description = "参数：用户id请求体，权限：所有人，方法名：listUserOrder")
    public BaseResponse<List<OrderVO>> listUserOrder(@RequestBody CommonRequest commonRequest) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", id);
        List<Order> orders = orderService.list(queryWrapper);
        List<OrderVO> orderVOS = orders.stream().map(order -> {
            Long orderId = order.getId();
            QueryWrapper<OrderItems> orderItemsQueryWrapper = new QueryWrapper<>();
            orderItemsQueryWrapper.eq("orderId", orderId);
            List<OrderItems> orderItemsList = orderItemService.list(orderItemsQueryWrapper);
            OrderVO orderVO = new OrderVO();
            BeanUtils.copyProperties(order, orderVO);
            orderVO.setOrderItemList(orderItemsList);
            return orderVO;
        }).toList();
        return ResultUtils.success(orderVOS);
    }


    /**
     * 更新用户订单状态
     * @param orderUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新订单状态", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)，方法名：updateOrderStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateOrderStatus(@RequestBody OrderUpdateRequest orderUpdateRequest) {
        if (orderUpdateRequest == null || orderUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderUpdateRequest.getId();
        String orderStatus = orderUpdateRequest.getOrderStatus();
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        boolean result = orderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


}
