package com.cultural.heritage.controller.user;


import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.config.WxOpenConfig;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.user.*;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.vo.UserVO;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static com.cultural.heritage.constant.UserConstant.SALT;

/**
 * 用户接口
 * @author 陈新知
 */
@Slf4j
@RestController
@Tag(name = "用户管理模块")
@RequestMapping("/user")
public class UserController {


    @Resource
    private UserService userService;


    @Resource
    private WxOpenConfig wxOpenConfig;


    /**
     * 用户登录
     * @param userLoginRequest  用户登录请求体
     * @param request           http
     * @return 登录用户信息
     */
    @PostMapping("/login")
    @Operation(summary = "Web端用户登录", description = "参数：{账号，密码}, 权限：所有人), 方法名：userLogin")
    public BaseResponse<UserVO> userLogin(@RequestBody UserLoginRequest userLoginRequest, HttpServletRequest request){
        if(userLoginRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        String userAccount = userLoginRequest.getUserAccount();
        String userPassword = userLoginRequest.getUserPassword();
        if(StringUtils.isAnyBlank(userAccount, userPassword)){
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        UserVO userVO = userService.userLogin(userAccount, userPassword, request);
        return ResultUtils.success(userVO);
    }




    /**
     * 用户登录（微信小程序）
     * @param code      用户登录凭证
     * @param request   http
     * @return          用户登录信息
     */
    @GetMapping("/login/wx_open")
    @Operation(summary = "(无法测)微信小程序用户登录", description = "参数：code(用户登录凭证), 权限：所有人, 方法名：userLoginByWxOpen")
    public BaseResponse<UserVO> userLoginByWxOpen(@RequestParam("code") String code, HttpServletRequest request) {
        WxMaJscode2SessionResult sessionInfo;
        try {
            WxMaService wxMaService = wxOpenConfig.getWxMaService();
            sessionInfo = wxMaService.jsCode2SessionInfo(code);
            String openid = sessionInfo.getOpenid();
            if (StringUtils.isAnyBlank(openid)) {
                throw new BusinessException(ErrorCode.SYSTEM_ERROR);
            }
            return ResultUtils.success(userService.userLoginByMpOpen(sessionInfo, request));
        } catch (WxErrorException e) {
            log.error("userLoginByWxOpen error", e);
            throw new BusinessException(ErrorCode.SYSTEM_ERROR, "登录失败，系统错误");
        }
    }


    /**
     * 退出登录
     * @param request http
     * @return 是否退出登录成功
     */
    @PostMapping("/logout")
    @Operation(summary = "Web端管理员退出登录", description = "参数：无，权限：所有人，方法名：userLogout")
    public BaseResponse<Boolean> userLogout(HttpServletRequest request) {
        boolean result = userService.userLogout(request);
        return ResultUtils.success(result);
    }


    /**
     * 获取当前登录用户
     * @param request   http
     * @return  用户登录信息
     */
    @GetMapping("/get/login")
    @Operation(summary = "小程序端用户获取登录信息", description = "参数：无，权限：所有人, 方法名：getLoginUser")
    public BaseResponse<UserVO> getLoginUser(HttpServletRequest request) {
        User user = userService.getLoginUser(request);
        return ResultUtils.success(userService.getUserVO(user));
    }



    /**
     * 更新个人信息
     * @param userUpdateMyRequest 更新请求体
     * @param request             http
     * @return  是否更新成功
     */
    @PostMapping("/update/my")
    @Operation(summary = "小程序端用户更新个人信息", description = "参数：用户更新请求体，权限：所有人, 方法名：updateMyUser")
    public BaseResponse<Boolean> updateMyUser(@RequestBody UserUpdateMyRequest userUpdateMyRequest, HttpServletRequest request) {
        if (userUpdateMyRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User loginUser = userService.getLoginUser(request);
        User user = new User();
        BeanUtils.copyProperties(userUpdateMyRequest, user);
        user.setId(loginUser.getId());
        boolean result = userService.updateById(user);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 创建用户
     * @param userAddRequest 用户添加请求体
     * @return 添加用户的信息
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员创建用户", description = "参数：用户添加请求体, 权限：管理员(boss), 方法名：addUser")
    @AuthCheck(mustRole = UserConstant.BOSS_ROLE)
    public BaseResponse<User> addUser(@RequestBody UserAddRequest userAddRequest) {
        if (userAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User user = new User();
        BeanUtils.copyProperties(userAddRequest, user);
        String encryptPassword = DigestUtils.md5DigestAsHex((SALT + user.getUserPassword()).getBytes());
        user.setUserPassword(encryptPassword);
        boolean save = userService.save(user);
        if (!save) {
            throw new BusinessException(ErrorCode.SYSTEM_ERROR);
        }
        return ResultUtils.success(user, "添加用户成功");
    }


    /**
     * 删除用户
     * @param deleteRequest 用户删除请求体
     * @return 是否删除
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除用户", description = "参数：用户删除请求体, 权限：管理员(boss), 方法名：deleteUser")
    @AuthCheck(mustRole = UserConstant.BOSS_ROLE)
    public BaseResponse<Boolean> deleteUser(@RequestBody CommonRequest deleteRequest) {
        if (deleteRequest == null || deleteRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        boolean result = userService.removeById(deleteRequest.getId());
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 更新用户
     * @param userUpdateRequest 用户更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新用户", description = "参数：用户更新请求体, 权限：管理员(boss), 方法名：updateUser")
    @AuthCheck(mustRole = UserConstant.BOSS_ROLE)
    public BaseResponse<Boolean> updateUser(@RequestBody UserUpdateRequest userUpdateRequest) {
        if (userUpdateRequest == null || userUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User user = new User();
        BeanUtils.copyProperties(userUpdateRequest, user);
        boolean result = userService.updateById(user);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true);
    }


    /**
     * 根据 id 获取用户
     * @param id
     * @return 用户信息
     */
    @GetMapping("/get")
    @Operation(summary = "(未使用)Web端管理员根据id获取用户信息", description = "参数：用户id, 权限：管理员(admin, boss), 方法名：getUserById")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<User> getUserById(long id) {
        if (id <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        User user = userService.getById(id);
        ThrowUtils.throwIf(user == null, ErrorCode.NOT_FOUND_ERROR);
        return ResultUtils.success(user);
    }


    /**
     * 获取用户数量
     */
    @GetMapping("/count")
    @Operation(summary = "(未使用)Web端管理员获取用户数量", description = "参数：无, 权限：管理员(admin, boss), 方法名：getUserCount")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Long> getUserCount() {
        QueryWrapper<User> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userRole", "user");
        long count = userService.count(queryWrapper);
        return ResultUtils.success(count);
    }



    /**
     * 分页获取用户列表
     * @param userQueryRequest  查询用户请求体
     * @return  用户信息列表
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页获取用户信息", description = "参数：用户查询请求体, 排序字段 sortField: points id，权限：管理员(admin, boss), 方法名：listUserByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<User>> listUserByPage(@RequestBody UserQueryRequest userQueryRequest) {
        long current = userQueryRequest.getCurrent();
        long pageSize = userQueryRequest.getPageSize();
        QueryWrapper<User> queryWrapper = userService.getQueryWrapper(userQueryRequest);
        Page<User> page = userService.page(new Page<>(current, pageSize), queryWrapper);
        return ResultUtils.success(page);
    }



    /**
     * 分页获取用户封装列表
     *
     * @param userQueryRequest  查询用户请求体
     * @return  脱敏用户信息列表
     */
    @PostMapping("/list/page/vo")
    @Operation(summary = "(未使用)Web端管理员分页获取脱敏用户信息", description = "参数：用户查询请求体, 排序字段 sortField: points id， 权限：管理员(admin, boss), 方法名：listUserVOByPage")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<UserVO>> listUserVOByPage(@RequestBody UserQueryRequest userQueryRequest) {
        if (userQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = userQueryRequest.getCurrent();
        long pageSize = userQueryRequest.getPageSize();
//        ThrowUtils.throwIf(size > 20, ErrorCode.PARAMS_ERROR);
        QueryWrapper<User> queryWrapper = userService.getQueryWrapper(userQueryRequest);
        Page<User> userPage = userService.page(new Page<>(current, pageSize), queryWrapper);

        Page<UserVO> userVOPage = new Page<>(current, pageSize, userPage.getTotal());
        List<UserVO> userVO = userService.getUserVO(userPage.getRecords());

        userVOPage.setRecords(userVO);
        return ResultUtils.success(userVOPage);
    }






//    /**
//     * 图形验证码
//     * @return
//     */
//    @GetMapping("/captcha")
//    public BaseResponse<Map> getCapthca(){
//        //设置生成的长宽还有数字位数
//        SpecCaptcha specCaptcha = new SpecCaptcha(130, 48, 4);
//        String code = specCaptcha.text().toLowerCase();
//        String uuid = IdUtil.simpleUUID();
//        redisTemplate.opsForValue().set(uuid,code,120, TimeUnit.SECONDS);
//        HashMap<String, String> map = new HashMap<>(3);
//        map.put("uuid",uuid);
//        map.put("code",code);
//        map.put("captcha",specCaptcha.toBase64());
//        return ResultUtils.success(map,"成功");
//    }





}
